/**
	@file libtb-cstrans.c
	@brief e[uWϊ
	@author Atsushi WATANABE <atusi_w@roboken.esys.tsukuba.ac.jp> <atusi_w@doramanjyu.com> 
 */

#define _XOPEN_SOURCE
#define _BSD_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <omp.h>


#include "tb-cstrans.h"

CSTrans *cstrans_init( int resolution )
{
	CSTrans *table;
	int i;
	double interval;
	double angle;

	table = malloc( sizeof( CSTrans ) );
	table->sin = malloc( sizeof( float ) * resolution );
	table->cos = malloc( sizeof( float ) * resolution );
	table->resolution = resolution;

	interval = 2 * M_PI / (double)resolution;
	angle = 0;

	for( i = 0; i < resolution; i ++ ){
		angle += interval;
		table->sin[i] = (float)sin( angle );
		table->cos[i] = (float)cos( angle );
	}

	return table;
}

void cstrans_free( CSTrans *table )
{
	free( table->sin );
	free( table->cos );
	free( table );
}

void cstrans_polar2rect( CSTrans *cst, vector *src, vector *dest, int len )
{	// 15us?
	int i;
	float resp2pi;

	resp2pi = (float)cst->resolution / ( 2 * M_PI );

	for( i = 0; i < len; i ++ ){
		int a;
		a = (int)( resp2pi * src[ i ].theta );
		while( a < 0 ) a += cst->resolution;
		while( a >= cst->resolution ) a -= cst->resolution;

		dest[ i ].x = cst->cos[ a ] * src[ i ].r;
		dest[ i ].y = cst->sin[ a ] * src[ i ].r;
	}
}

void cstrans_rotate( CSTrans *cst, vector *src, vector *dest, int len, int step )
{
	// 10us
	int a;
	vector mat;
	vector dest_tmp1, dest_tmp2;
	int i;
	
	a = step;
	while( a < 0 ) a += cst->resolution;
	while( a >= cst->resolution ) a -= cst->resolution;

	mat.array[0] = cst->cos[ a ];
	mat.array[1] = cst->sin[ a ];

	for( i = 0; i < len; i += 2 ){
		dest_tmp1.x = mat.array[0] * src[ i ].x - mat.array[1] * src[ i ].y;
		dest_tmp1.y = mat.array[1] * src[ i ].x + mat.array[0] * src[ i ].y;
		dest[ i ] = dest_tmp1;
		dest_tmp2.x = mat.array[0] * src[ i+1 ].x - mat.array[1] * src[ i+1 ].y;
		dest_tmp2.y = mat.array[1] * src[ i+1 ].x + mat.array[0] * src[ i+1 ].y;
		dest[ i + 1 ] = dest_tmp2;
	}
}
